@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2012 Daniel Scharrer
+ * Copyright (C) 2011-2013 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -18,27 +18,28 @@
  * 3. This notice may not be removed or altered from any source distribution.
  */
 
-#include <fstream>
-#include <iostream>
-#include <iomanip>
-#include <string>
 #include <algorithm>
-#include <cstring>
-#include <vector>
-#include <map>
 #include <cctype>
+#include <cstring>
+#include <cstdlib>
+#include <iomanip>
+#include <iostream>
+#include <map>
+#include <string>
+#include <vector>
 
+#include <boost/algorithm/string/predicate.hpp>
 #include <boost/foreach.hpp>
 #include <boost/make_shared.hpp>
 #include <boost/program_options.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <boost/ptr_container/ptr_vector.hpp>
 #include <boost/filesystem/path.hpp>
-#include <boost/filesystem/fstream.hpp>
 #include <boost/filesystem/operations.hpp>
 
+#include "release.hpp"
+
 #include "cli/debug.hpp"
-#include "cli/version.hpp"
 
 #include "loader/offsets.hpp"
 
@@ -55,30 +56,90 @@
 
 #include "util/boostfs_compat.hpp"
 #include "util/console.hpp"
+#include "util/fstream.hpp"
 #include "util/load.hpp"
 #include "util/log.hpp"
 #include "util/output.hpp"
+#include "util/time.hpp"
+#include "util/windows.hpp"
 
-using std::cout;
-using std::string;
-using std::endl;
-using std::setw;
-using std::setfill;
-
-namespace io = boost::iostreams;
 namespace fs = boost::filesystem;
 namespace po = boost::program_options;
 
-static void print_version() {
-	std::cout << color::white << innoextract_version << color::reset
+
+enum ExitValues {
+	ExitSuccess = 0,
+	ExitUserError = 1,
+	ExitDataError = 2
+};
+
+
+static const char * get_command(const char * argv0) {
+	
+	if(!argv0) {
+		argv0 = innoextract_name;
+	}
+	std::string var = argv0;
+	
+#ifdef _WIN32
+	size_t pos = var.find_last_of("/\\");
+#else
+	size_t pos = var.find_last_of('/');
+#endif
+	if(pos != std::string::npos) {
+		var = var.substr(pos + 1);
+	}
+	
+	var += "_COMMAND";
+	
+	const char * env = std::getenv(var.c_str());
+	if(env) {
+		return env;
+	} else {
+		return argv0;
+	}
+}
+
+
+struct options {
+	
+	bool quiet;
+	bool silent;
+	
+	bool list; // The --list action has been explicitely specified
+	bool test; // The --test action has been explicit specified
+	bool extract; // The --extract action has been specified or automatically enabled
+	
+	bool preserve_file_times;
+	bool local_timestamps;
+	
+	std::string language;
+	
+	setup::filename_map filenames;
+	
+	fs::path output_dir;
+	
+};
+
+
+static void print_version(const options & o) {
+	if(o.silent) {
+		std::cout << innoextract_version << '\n';
+		return;
+	}
+	std::cout << color::white << innoextract_name
+	          << ' ' << innoextract_version << color::reset
 #ifdef DEBUG
 	          << " (with debug output)"
 #endif
 	          << '\n';
-	std::cout << "Extracts installers created by " << color::cyan
-	          << innosetup_versions << color::reset << '\n';
+	if(!o.quiet) {
+		std::cout << "Extracts installers created by " << color::cyan
+		          << innosetup_versions << color::reset << '\n';
+	}
 }
 
+
 static void print_help(const char * name, const po::options_description & visible) {
 	std::cout << color::white << "Usage: " << name << " [options] <setup file(s)>\n\n"
 	          << color::reset;
@@ -88,51 +149,58 @@ static void print_help(const char * name, const po::options_description & visibl
 	std::cout << "Extracts installers created by " << color::cyan
 	          << innosetup_versions << color::reset << '\n';
 	std::cout << '\n';
-	std::cout << color::white << innoextract_version << color::reset
+	std::cout << color::white << innoextract_name
+	          << ' ' << innoextract_version << color::reset
 	          << ' ' << innoextract_copyright << '\n';
 	std::cout << "This is free software with absolutely no warranty.\n";
 }
 
+
 static void print_license() {
 	
-	std::cout << color::white << innoextract_version << color::reset
+	std::cout << color::white << innoextract_name
+	          << ' ' << innoextract_version << color::reset
 	          << ' ' << innoextract_copyright << '\n';
-	std::cout << '\n';
-	std::cout << "This software is provided 'as-is', without any express or implied\n"
-	             "warranty.  In no event will the author(s) be held liable for any damages\n"
-	             "arising from the use of this software.\n"
-	             "\n"
-	             "Permission is granted to anyone to use this software for any purpose,\n"
-	             "including commercial applications, and to alter it and redistribute it\n"
-	             "freely, subject to the following restrictions:\n"
-	             "\n"
-	             "1. The origin of this software must not be misrepresented; you must not\n"
-	             "   claim that you wrote the original software. If you use this software\n"
-	             "   in a product, an acknowledgment in the product documentation would be\n"
-	             "   appreciated but is not required.\n"
-	             "2. Altered source versions must be plainly marked as such, and must not be\n"
-	             "   misrepresented as being the original software.\n"
-	             "3. This notice may not be removed or altered from any source distribution.\n"
+	std::cout << '\n'<< innoextract_license << '\n';
 	;
 }
 
-struct options {
+
+struct file_output {
 	
-	bool silent;
+	fs::path name;
+	util::ofstream stream;
 	
-	bool dump;
-	bool list;
-	bool test;
-	
-	std::string language;
-	
-	setup::filename_map filenames;
+	explicit file_output(const fs::path & file) : name(file) {
+		try {
+			fs::create_directories(name.parent_path());
+		} catch(...) {
+			throw std::runtime_error("error creating directories for \""
+			                         + name.string() + '"');
+		}
+		stream.open(name, std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
+		if(!stream.is_open()) {
+			throw std::runtime_error("error opening output file \"" + name.string() + '"');
+		}
+	}
 	
 };
 
+
 static void process_file(const fs::path & file, const options & o) {
 	
-	fs::ifstream ifs(file, std::ios_base::in | std::ios_base::binary | std::ios_base::ate);
+	bool is_directory;
+	try {
+		is_directory = fs::is_directory(file);
+	} catch(...) {
+		throw std::runtime_error("error opening file \"" + file.string()
+		                         + "\": access denied");
+	}
+	if(is_directory) {
+		throw std::runtime_error("input file \"" + file.string() + "\" is a directory");
+	}
+	
+	util::ifstream ifs(file, std::ios_base::in | std::ios_base::binary);
 	if(!ifs.is_open()) {
 		throw std::runtime_error("error opening file \"" + file.string() + '"');
 	}
@@ -140,37 +208,44 @@ static void process_file(const fs::path & file, const options & o) {
 	loader::offsets offsets;
 	offsets.load(ifs);
 	
-	cout << std::boolalpha;
+	std::cout << std::boolalpha;
 	
 #ifdef DEBUG
 	if(logger::debug) {
 		print_offsets(offsets);
-		cout << '\n';
+		std::cout << '\n';
+	}
+#endif
+	
+	setup::info::entry_types entries = setup::info::DataEntries | setup::info::Files;
+#ifdef DEBUG
+	if(logger::debug) {
+		entries = setup::info::entry_types::all();
 	}
 #endif
 	
 	ifs.seekg(offsets.header_offset);
 	setup::info info;
-	info.load(ifs);
+	info.load(ifs, entries);
 	
-	if(!logger::quiet) {
+	if(!o.quiet) {
 		const std::string & name = info.header.app_versioned_name.empty()
 		                           ? info.header.app_name : info.header.app_versioned_name;
-		cout << (o.list ? "Listing" : o.test ? "Testing" : "Extracting")
-		     << " \"" << color::green << name << color::reset
-		     << "\" - setup data version " << color::white << info.version << color::reset
-		     << std::endl;
+		std::cout << (o.extract ? "Extracting" : o.test ? "Testing" : "Listing")
+		          << " \"" << color::green << name << color::reset
+		          << "\" - setup data version " << color::white << info.version << color::reset
+		          << std::endl;
 	}
 	
 #ifdef DEBUG
 	if(logger::debug) {
-		cout << '\n';
+		std::cout << '\n';
 		print_info(info);
-		cout << '\n';
+		std::cout << '\n';
 	}
 #endif
 	
-	uint64_t total_size = 0;
+	boost::uint64_t total_size = 0;
 	
 	std::vector< std::vector<size_t> > files_for_location;
 	files_for_location.resize(info.data_entries.size());
@@ -193,9 +268,9 @@ static void process_file(const fs::path & file, const options & o) {
 	}
 	
 	boost::scoped_ptr<stream::slice_reader> slice_reader;
-	if(!o.list) {
+	if(o.extract || o.test) {
 		if(offsets.data_offset) {
-			slice_reader.reset(new stream::slice_reader(file, offsets.data_offset));
+			slice_reader.reset(new stream::slice_reader(&ifs, offsets.data_offset));
 		} else {
 			slice_reader.reset(new stream::slice_reader(file.parent_path(), file.stem(),
 			                                            info.header.slices_per_disk));
@@ -206,22 +281,26 @@ static void process_file(const fs::path & file, const options & o) {
 	
 	BOOST_FOREACH(const Chunks::value_type & chunk, chunks) {
 		
-		debug("[starting " << chunk.first.compression << " chunk @ " << chunk.first.first_slice
+		debug("[starting " << chunk.first.compression << " chunk @ slice " << chunk.first.first_slice
 		      << " + " << print_hex(offsets.data_offset) << " + " << print_hex(chunk.first.offset)
 		      << ']');
 		
-		stream::chunk_reader::pointer chunk_source;
-		if(!o.list) {
-			chunk_source = stream::chunk_reader::get(*slice_reader, chunk.first);
+		if(chunk.first.encrypted) {
+			log_warning << "skipping encrypted chunk (unsupported)";
+			continue;
 		}
 		
-		uint64_t offset = 0;
+		stream::chunk_reader::pointer chunk_source;
+		if(o.extract || o.test) {
+			chunk_source = stream::chunk_reader::get(*slice_reader, chunk.first);
+		}
+		boost::uint64_t offset = 0;
 		
 		BOOST_FOREACH(const Files::value_type & location, chunk.second) {
 			const stream::file & file = location.first;
 			
 			// Convert output filenames
-			typedef std::pair<fs::path, size_t> file_t;
+			typedef std::pair<std::string, size_t> file_t;
 			std::vector<file_t> output_names;
 			for(size_t i = 0; i < files_for_location[location.second].size(); i++) {
 				
@@ -234,16 +313,7 @@ static void process_file(const fs::path & file, const options & o) {
 				}
 				
 				if(!info.files[file_i].destination.empty()) {
-					fs::path path;
-					if(o.dump) {
-						std::string file = info.files[file_i].destination;
-						if(o.filenames.lowercase) {
-							std::transform(file.begin(), file.end(), file.begin(), ::tolower);
-						}
-						path = file;
-					} else {
-						path = o.filenames.convert(info.files[file_i].destination);
-					}
+					std::string path = o.filenames.convert(info.files[file_i].destination);
 					if(!path.empty()) {
 						output_names.push_back(std::make_pair(path, file_i));
 					}
@@ -258,7 +328,7 @@ static void process_file(const fs::path & file, const options & o) {
 			// Print filename and size
 			if(!o.silent) {
 				
-				progress::clear();
+				extract_progress.clear();
 				
 				std::cout << " - ";
 				bool named = false;
@@ -266,7 +336,7 @@ static void process_file(const fs::path & file, const options & o) {
 					if(named) {
 						std::cout << ", ";
 					}
-					std::cout << '"' << color::white << path.first.string() << color::reset << '"';
+					std::cout << '"' << color::white << path.first << color::reset << '"';
 					if(!info.files[path.second].languages.empty()) {
 						std::cout << " [" << color::green << info.files[path.second].languages
 						          << color::reset << "]";
@@ -276,21 +346,32 @@ static void process_file(const fs::path & file, const options & o) {
 				if(!named) {
 					std::cout << color::white << "unnamed file" << color::reset;
 				}
-				if(!logger::quiet) {
-				#ifdef DEBUG
-					std::cout << " @ " << print_hex(file.offset);
-				#endif
-				std::cout << " (" << color::dim_cyan << print_bytes(file.size)
-				          << color::reset << ")";
+				if(!o.quiet) {
+					if(logger::debug) {
+						std::cout << " @ " << print_hex(file.offset);
+					}
+					std::cout << " (" << color::dim_cyan << print_bytes(file.size)
+					          << color::reset << ")";
 				}
 				std::cout << '\n';
-				
-				if(!o.list) {
-					extract_progress.update(0, true);
+				if(o.extract || o.test) {
+					std::cout.flush();
 				}
+				
+				extract_progress.update(0, true);
+				
+			} else if(o.list) {
+				extract_progress.clear();
+				BOOST_FOREACH(const file_t & path, output_names) {
+					std::cout << color::white << path.first << color::reset << '\n';
+				}
+				if(o.extract || o.test) {
+					std::cout.flush();
+				}
+				extract_progress.update(0, true);
 			}
 			
-			if(o.list) {
+			if(!o.extract && !o.test) {
 				continue;
 			}
 			
@@ -301,7 +382,7 @@ static void process_file(const fs::path & file, const options & o) {
 			}
 			if(file.offset > offset) {
 				debug("discarding " << print_bytes(file.offset - offset));
-				discard(*chunk_source, file.offset - offset);
+				util::discard(*chunk_source, file.offset - offset);
 			}
 			offset = file.offset + file.size;
 			
@@ -312,44 +393,53 @@ static void process_file(const fs::path & file, const options & o) {
 			file_source = stream::file_reader::get(*chunk_source, file, &checksum);
 			
 			// Open output files
-			boost::ptr_vector<fs::ofstream> output;
+			boost::ptr_vector<file_output> output;
 			if(!o.test) {
 				output.reserve(output_names.size());
 				BOOST_FOREACH(const file_t & path, output_names) {
-					try {
-						fs::create_directories(path.first.parent_path());
-					} catch(...) {
-						throw std::runtime_error("error creating directories for \""
-						                         + path.first.string() + '"');
-					}
-					output.push_back(new fs::ofstream(path.first));
-					if(!output.back().is_open()) {
-						throw std::runtime_error("error opening output file \""
-						                        + path.first.string() + '"');
-					}
+					output.push_back(new file_output(o.output_dir / path.first));
 				}
 			}
 			
 			// Copy data
 			while(!file_source->eof()) {
 				char buffer[8192 * 10];
-				std::streamsize n = file_source->read(buffer, ARRAY_SIZE(buffer)).gcount();
+				std::streamsize buffer_size = std::streamsize(ARRAY_SIZE(buffer));
+				std::streamsize n = file_source->read(buffer, buffer_size).gcount();
 				if(n > 0) {
-					
-					BOOST_FOREACH(fs::ofstream & ofs, output) {
-						ofs.write(buffer, n);
+					BOOST_FOREACH(file_output & out, output) {
+						out.stream.write(buffer, n);
+						if(out.stream.fail()) {
+							throw std::runtime_error("error writing file \""
+							                         + out.name.string() + '"');
+						}
 					}
-					
-					extract_progress.update(uint64_t(n));
+					extract_progress.update(boost::uint64_t(n));
 				}
 			}
 			
+			// Adjust file timestamps
+			if(o.preserve_file_times) {
+				const setup::data_entry & data = info.data_entries[location.second];
+				util::time filetime = data.timestamp;
+				if(o.local_timestamps && !(data.options & data.TimeStampInUTC)) {
+					filetime = util::to_local_time(filetime);
+				}
+				BOOST_FOREACH(file_output & out, output) {
+					out.stream.close();
+					if(!util::set_file_time(out.name, filetime, data.timestamp_nsec)) {
+						log_warning << "error setting timestamp on file " << out.name;
+					}
+				}
+			}
+			
+			// Verify checksums
 			if(checksum != file.checksum) {
 				log_warning << "checksum mismatch:\n"
 				            << "actual:   " << checksum << '\n'
 				            << "expected: " << file.checksum;
 				if(o.test) {
-					throw new std::runtime_error("integrity test failed");
+					throw std::runtime_error("integrity test failed");
 				}
 			}
 		}
@@ -358,6 +448,7 @@ static void process_file(const fs::path & file, const options & o) {
 	extract_progress.clear();
 }
 
+
 int main(int argc, char * argv[]) {
 	
 	::options o;
@@ -365,7 +456,7 @@ int main(int argc, char * argv[]) {
 	po::options_description generic("Generic options");
 	generic.add_options()
 		("help,h", "Show supported options")
-		("version,v", "Print the version information")
+		("version,v", "Print version information")
 		("license", "Show license information")
 	;
 	
@@ -376,14 +467,16 @@ int main(int argc, char * argv[]) {
 		("list,l", "Only list files, don't write anything")
 	;
 	
-	po::options_description filter("Filters");
+	po::options_description filter("Modifiers");
 	filter.add_options()
 		("dump", "Dump contents without converting filenames")
 		("lowercase,L", "Convert extracted filenames to lower-case")
 		("language", po::value<std::string>(), "Extract files for the given language")
+		("timestamps,T", po::value<std::string>(), "Timezone for file times or \"local\" or \"none\"")
+		("output-dir,d", po::value<fs::path>(), "Extract files into the given directory")
 	;
 	
-	po::options_description io("I/O options");
+	po::options_description io("Display options");
 	io.add_options()
 		("quiet,q", "Output less information")
 		("silent,s", "Output only error/warning information")
@@ -396,7 +489,7 @@ int main(int argc, char * argv[]) {
 	
 	po::options_description hidden("Hidden options");
 	hidden.add_options()
-		("setup-files", po::value< std::vector<string> >(), "Setup files to be extracted")
+		("setup-files", po::value< std::vector<std::string> >(), "Setup files to be extracted")
 		/**/;
 	
 	po::options_description options_desc;
@@ -417,13 +510,14 @@ int main(int argc, char * argv[]) {
 		po::notify(options);
 	} catch(po::error & e) {
 		std::cerr << "Error parsing command-line: " << e.what() << "\n\n";
-		print_help(argv[0], visible);
-		return 1;
+		print_help(get_command(argv[0]), visible);
+		return ExitUserError;
 	}
 	
 	// Verbosity settings.
-	o.silent = options.count("silent");
-	logger::quiet = o.silent || options.count("quiet");
+	o.silent = (options.count("silent") != 0);
+	o.quiet = o.silent || options.count("quiet");
+	logger::quiet = o.quiet;
 #ifdef DEBUG
 	if(options.count("debug")) {
 		logger::debug = true;
@@ -448,55 +542,94 @@ int main(int argc, char * argv[]) {
 	color::init(color_e, progress_e);
 	
 	// Help output.
-	if(options.count("help")) {
-		print_help(argv[0], visible);
-		return 0;
+	if(options.count("help") != 0) {
+		print_help(get_command(argv[0]), visible);
+		return ExitSuccess;
 	}
 	
 	// License output
-	if(options.count("license")) {
+	if(options.count("license") != 0) {
 		print_license();
-		return 0;
+		return ExitSuccess;
 	}
 	
 	// Main action.
-	o.list = options.count("list");
-	bool extract = options.count("extract");
-	o.test = options.count("test");
-	bool explicit_action = o.list || extract || o.test;
+	o.list = (options.count("list") != 0);
+	o.extract = (options.count("extract") != 0);
+	o.test = (options.count("test") != 0);
+	bool explicit_action = o.list || o.test || o.extract;
 	if(!explicit_action) {
-		extract = true;
+		o.extract = true;
 	}
-	if(o.list + extract + o.test > 1) {
+	if(o.extract + o.test > 1) {
 		log_error << "cannot specify multiple actions";
-		return 0;
+		return ExitUserError;
+	}
+	if(!o.extract && !o.test) {
+		progress::set_enabled(false);
 	}
 	
 	// Additional actions.
-	o.dump = options.count("dump");
-	o.filenames.set_lowercase(options.count("lowercase"));
+	o.filenames.set_expand(options.count("dump") == 0);
+	o.filenames.set_lowercase(options.count("lowercase") != 0);
+	
+	// File timestamps
+	{
+		o.preserve_file_times = true, o.local_timestamps = false;
+		po::variables_map::const_iterator i = options.find("timestamps");
+		if(i != options.end()) {
+			std::string timezone = i->second.as<std::string>();
+			if(boost::iequals(timezone, "none")) {
+				o.preserve_file_times = false;
+			} else if(!boost::iequals(timezone, "UTC")) {
+				o.local_timestamps = true;
+				if(!boost::iequals(timezone, "local")) {
+					util::set_local_timezone(timezone);
+				}
+			}
+		}
+	}
 	
 	// List version.
-	if(options.count("version")) {
-		print_version();
+	if(options.count("version") != 0) {
+		print_version(o);
 		if(!explicit_action) {
-			return 0;
+			return ExitSuccess;
 		}
 	}
 	
-	po::variables_map::const_iterator i = options.find("language");
-	if(i != options.end()) {
-		o.language = i->second.as<std::string>();
+	{
+		po::variables_map::const_iterator i = options.find("language");
+		if(i != options.end()) {
+			o.language = i->second.as<std::string>();
+		}
 	}
 	
-	if(!options.count("setup-files")) {
+	if(!options.count("setup-files") != 0) {
 		if(!o.silent) {
-			std::cout << "no input files specified\n";
+			std::cout << get_command(argv[0]) << ": no input files specified\n";
+			std::cout << "Try the --help (-h) option for usage information.\n";
 		}
-		return 0;
+		return ExitSuccess;
 	}
 	
-	const std::vector<string> & files = options["setup-files"].as< std::vector<string> >();
+	{
+		po::variables_map::const_iterator i = options.find("output-dir");
+		if(i != options.end()) {
+			o.output_dir = i->second.as<fs::path>();
+			try {
+				if(!fs::exists(o.output_dir)) {
+					fs::create_directory(o.output_dir);
+				}
+			} catch(...) {
+				log_error << "could not create output directory " << o.output_dir;
+				return ExitDataError;
+			}
+		}
+	}
+	
+	const std::vector<std::string> & files = options["setup-files"]
+	                                         .as< std::vector<std::string> >();
 	
 	try {
 		BOOST_FOREACH(const std::string & file, files) {
@@ -532,5 +665,5 @@ int main(int argc, char * argv[]) {
 		std::cout << '.' << std::endl;
 	}
 	
-	return logger::total_errors == 0 ? 0 : 1;
+	return logger::total_errors == 0 ? ExitSuccess : ExitDataError;
 }
