@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Daniel Scharrer
+ * Copyright (C) 2011-2013 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -20,9 +20,9 @@
 
 #include "loader/offsets.hpp"
 
-#include <stdint.h>
 #include <cstring>
 
+#include <boost/cstdint.hpp>
 #include <boost/static_assert.hpp>
 
 #include <stddef.h>
@@ -57,8 +57,8 @@ const setup_loader_version known_setup_loader_versions[] = {
 
 const int ResourceNameInstaller = 11111;
 
-const uint32_t SetupLoaderHeaderOffset = 0x30;
-const uint32_t SetupLoaderHeaderMagic = 0x6f6e6e49;
+const boost::uint32_t SetupLoaderHeaderOffset = 0x30;
+const boost::uint32_t SetupLoaderHeaderMagic = 0x6f6e6e49;
 
 } // anonymous namespace
 
@@ -66,14 +66,14 @@ bool offsets::load_from_exe_file(std::istream & is) {
 	
 	is.seekg(SetupLoaderHeaderOffset);
 	
-	uint32_t magic = load_number<uint32_t>(is);
+	boost::uint32_t magic = util::load<boost::uint32_t>(is);
 	if(is.fail() || magic != SetupLoaderHeaderMagic) {
 		is.clear();
 		return false;
 	}
 	
-	uint32_t offset_table_offset = load_number<uint32_t>(is);
-	uint32_t not_offset_table_offset = load_number<uint32_t>(is);
+	boost::uint32_t offset_table_offset = util::load<boost::uint32_t>(is);
+	boost::uint32_t not_offset_table_offset = util::load<boost::uint32_t>(is);
 	if(is.fail() || offset_table_offset != ~not_offset_table_offset) {
 		is.clear();
 		return false;
@@ -93,7 +93,7 @@ bool offsets::load_from_exe_resource(std::istream & is) {
 	return load_offsets_at(is, resource.offset);
 }
 
-bool offsets::load_offsets_at(std::istream & is, uint32_t pos) {
+bool offsets::load_offsets_at(std::istream & is, boost::uint32_t pos) {
 	
 	if(is.seekg(pos).fail()) {
 		is.clear();
@@ -101,14 +101,15 @@ bool offsets::load_offsets_at(std::istream & is, uint32_t pos) {
 	}
 	
 	char magic[12];
-	if(is.read(magic, ARRAY_SIZE(magic)).fail()) {
+	if(is.read(magic, std::streamsize(ARRAY_SIZE(magic))).fail()) {
 		is.clear();
 		return false;
 	}
 	
 	setup::version_constant version = 0;
 	for(size_t i = 0; i < ARRAY_SIZE(known_setup_loader_versions); i++) {
-		BOOST_STATIC_ASSERT(ARRAY_SIZE(known_setup_loader_versions[i].magic) == ARRAY_SIZE(magic));
+		BOOST_STATIC_ASSERT(ARRAY_SIZE(known_setup_loader_versions[i].magic)
+		                    == ARRAY_SIZE(magic));
 		if(!memcmp(magic, known_setup_loader_versions[i].magic, ARRAY_SIZE(magic))) {
 			version = known_setup_loader_versions[i].version;
 			break;
@@ -123,40 +124,40 @@ bool offsets::load_offsets_at(std::istream & is, uint32_t pos) {
 	checksum.update(magic, ARRAY_SIZE(magic));
 	
 	if(version >= INNO_VERSION(5, 1,  5)) {
-		uint32_t revision = checksum.load_number<uint32_t>(is);
+		boost::uint32_t revision = checksum.load<boost::uint32_t>(is);
 		if(is.fail() || revision != 1) {
 			is.clear();
 			return false;
 		}
 	}
 	
-	(void)checksum.load_number<uint32_t>(is);
-	exe_offset = checksum.load_number<uint32_t>(is);
+	(void)checksum.load<boost::uint32_t>(is);
+	exe_offset = checksum.load<boost::uint32_t>(is);
 	
 	if(version >= INNO_VERSION(4, 1, 6)) {
 		exe_compressed_size = 0;
 	} else {
-		exe_compressed_size = checksum.load_number<uint32_t>(is);
+		exe_compressed_size = checksum.load<boost::uint32_t>(is);
 	}
 	
-	exe_uncompressed_size = checksum.load_number<uint32_t>(is);
+	exe_uncompressed_size = checksum.load<boost::uint32_t>(is);
 	
 	if(version >= INNO_VERSION(4, 0, 3)) {
 		exe_checksum.type = crypto::CRC32;
-		exe_checksum.crc32 = checksum.load_number<uint32_t>(is);
+		exe_checksum.crc32 = checksum.load<boost::uint32_t>(is);
 	} else {
 		exe_checksum.type = crypto::Adler32;
-		exe_checksum.adler32 = checksum.load_number<uint32_t>(is);
+		exe_checksum.adler32 = checksum.load<boost::uint32_t>(is);
 	}
 	
 	if(version >= INNO_VERSION(4, 0, 0)) {
 		message_offset = 0;
 	} else {
-		message_offset = load_number<uint32_t>(is);
+		message_offset = util::load<boost::uint32_t>(is);
 	}
 	
-	header_offset = checksum.load_number<uint32_t>(is);
-	data_offset = checksum.load_number<uint32_t>(is);
+	header_offset = checksum.load<boost::uint32_t>(is);
+	data_offset = checksum.load<boost::uint32_t>(is);
 	
 	if(is.fail()) {
 		is.clear();
@@ -164,7 +165,7 @@ bool offsets::load_offsets_at(std::istream & is, uint32_t pos) {
 	}
 	
 	if(version >= INNO_VERSION(4, 0, 10)) {
-		uint32_t expected = load_number<uint32_t>(is);
+		boost::uint32_t expected = util::load<boost::uint32_t>(is);
 		if(is.fail()) {
 			is.clear();
 			return false;
